// This is -*- C++ -*-
// $Id: color.h,v 1.4 1998/07/08 17:47:22 trow Exp $

/* 
 * color.h
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#ifndef _INC_COLOR_H
#define _INC_COLOR_H

#include <math.h>
#include <gdk/gdk.h>

class Gdk_Color {
public:
  Gdk_Color() { }
  Gdk_Color(const gchar* s) { set(s); }
  Gdk_Color(const Gdk_Color& cc) : c(cc.c) { }

  operator GdkColor*() { return &c; }
  GdkColor* color() { return &c; }

  void set_grey(gushort g) {
    c.red = c.green = c.blue = g;
    calc_pixel();
  }

  void set_grey_p(gdouble g) {
    c.red = c.green = c.blue = (gushort)(g*65535);
    calc_pixel();
  }

  void set_rgb(gushort red, gushort green, gushort blue) {
    c.red = red;
    c.green = green;
    c.blue = blue;
    calc_pixel();
  }

  void set_rgb_p(gdouble red, gdouble green, gdouble blue) {
    c.red = (gushort)(red*65535);
    c.green = (gushort)(green*65535);
    c.blue = (gushort)(blue*65535);
    calc_pixel();
  }

  void set_hsv(gdouble h, gdouble s, gdouble v);
  void set_hsl(gdouble h, gdouble s, gdouble l);

  void set(const gchar* s) {
    gdk_color_parse(s, &c);
    calc_pixel();
  }

  gushort red() const { return c.red; }
  gushort green() const { return c.green; }
  gushort blue() const { return c.blue; }

  gdouble red_p() const { return c.red / 65535.; }
  gdouble green_p() const { return c.green / 65535.; }
  gdouble blue_p() const { return c.blue / 65535.; }

  /// Gratuitous code-bloat begins here 

  void set_random(); // pointless, but fun
  gdouble distance_rgb_euclidean(const Gdk_Color& c) const;
  gdouble distance_rgb_taxicab(const Gdk_Color& c) const;

private:
  /*
    Notice that we have a hook in here for people who want to use their
    own color map, but it isn't really accessible.  I run X at 24bpp, and
    am selfishly not interested in figuring out stuff involving
    color indexing.

    At very least, there should be a static Gdk_Color::set_default_colormap()
    function and some way of using a different colormap on a local basis.
    None of this is hard to do, but I'm hoping that someone else will
    figure it out.  I've given you the source... what more do you need? 
   */
  void calc_pixel() {
    gdk_color_alloc(colormap ? colormap : gdk_colormap_get_system(), &c);
  }

  GdkColor c;
  static GdkColormap* colormap;
};

#endif // _INC_COLOR_H

// $Id: color.h,v 1.4 1998/07/08 17:47:22 trow Exp $
