// This is -*- C++ -*-
// $Id: gc.cc,v 1.6 1998/07/07 15:00:16 trow Exp $

/* 
 * gc.cc
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <string.h>
#include <X11/Xlib.h>
#include <X11/Xos.h>
#include <gdk/gdkprivate.h>
#include "gc.h"

Gdk_GC::Gdk_GC(GdkWindow* win) : window_(win), gc_(0)
{ }

Gdk_GC::Gdk_GC(Gtk_Widget* widget) :
  window_(GTK_WIDGET(widget->gtkobject)->window), gc_(0)
{ }

Gdk_GC::Gdk_GC(GdkWindow* win, GdkGC* gc) : 
  window_(win), gc_(gc)
{
  gdk_gc_ref(gc);
}

Gdk_GC::Gdk_GC(const Gdk_GC& gc) : window_(gc.window_), gc_(0)
{
  if (gc.gc_)
    gdk_gc_copy(gc_ = gdk_gc_new(window_), gc.gc_);
}

Gdk_GC::~Gdk_GC()
{
  if (gc_)
    gdk_gc_unref(gc_);
}

GdkFont*
Gdk_GC::font()
{
  GdkGCValues val;
  gdk_gc_get_values(gc(), &val);
  if (val.font == 0) {
    set_font("fixed");
    return font();  // If we don't have "fixed", this could get ugly.
  }
  return val.font;
}

gint
Gdk_GC::text_width(const gchar* s)
{
  gchar* ss = strdup(s);
  gchar* ss_orig = ss;

  gint width_max = 0;

  while (*ss) {
    gchar* p = ss;
    while (*p && *p != '\n') ++p;
    bool more_to_do = *p;
    *p = '\0';
    gint w = string_width(ss);
    if (w > width_max)
      width_max = w;
    ss = p;
    if (more_to_do) ++ss;
  }
 
  free(ss_orig);

  return width_max;
}

gint
Gdk_GC::text_height(const gchar* s)
{
  gint lines=0;
  while (*s) {
    ++lines;
    while (*s && *s != '\n') ++s;
    if (*s) ++s;
  }

  return lines * font_height();
}

void
Gdk_GC::set_line_width(gint line_width)
{
  GdkGCPrivate* gc_private = (GdkGCPrivate*)gc();
  XGCValues xvalues;
  xvalues.line_width = line_width;
  XChangeGC(gc_private->xdisplay, gc_private->xgc, GCLineWidth, &xvalues);
}

void
Gdk_GC::set_line_style(GdkLineStyle line_style)
{
  GdkGCPrivate* gc_private = (GdkGCPrivate*)gc();
  XGCValues xvalues;
  xvalues.line_style = line_style;
  XChangeGC(gc_private->xdisplay, gc_private->xgc, GCLineStyle, &xvalues);
}

void
Gdk_GC::set_cap_style(GdkCapStyle cap_style)
{
  GdkGCPrivate* gc_private = (GdkGCPrivate*)gc();
  XGCValues xvalues;
  xvalues.cap_style = cap_style;
  XChangeGC(gc_private->xdisplay, gc_private->xgc, GCCapStyle, &xvalues);
}

void
Gdk_GC::set_join_style(GdkJoinStyle join_style)
{
  GdkGCPrivate* gc_private = (GdkGCPrivate*)gc();
  XGCValues xvalues;
  xvalues.join_style = join_style;
  XChangeGC(gc_private->xdisplay, gc_private->xgc, GCJoinStyle, &xvalues);
}

void
Gdk_GC::set_dashes(gint dash_offset, char* dash_list, gint dash_list_len) {
  GdkGCPrivate* gc_private = (GdkGCPrivate*)gc();
  XSetDashes(gc_private->xdisplay, gc_private->xgc,
	     dash_offset, dash_list, dash_list_len);
}

void
Gdk_GC::set_dashes(gint size)
{
  char d[2] = { size, size };
  set_dashes(0,d,2);
}

void
Gdk_GC::set_dashes(gint on, gint off)
{
  char d[2] = { on, off };
  set_dashes(0,d,2);
}



// $Id: gc.cc,v 1.6 1998/07/07 15:00:16 trow Exp $
