// This is -*- C++ -*-
// $Id: gc.h,v 1.7 1998/07/21 19:41:27 hp Exp $

/* 
 * gc.h
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */


#ifndef _INC_GC_H
#define _INC_GC_H

#ifdef index
#undef index
#endif

#include <gtk--.h>
#include <gdk/gdk.h>
#include "color.h"

#include <iostream.h>

class Gdk_GC {
public:
  Gdk_GC(GdkWindow*);
  Gdk_GC(Gtk_Widget*);
  // Wrap an existing GdkGC; better make sure the window corresponds.
  Gdk_GC(GdkWindow*,GdkGC*); 
  Gdk_GC(const Gdk_GC&);
  ~Gdk_GC();

  // Hacks for getting at the underlying object, if you are so inclined.
  operator GdkGC*() { return gc(); }
  GdkGC* gc() {
    if (gc_ == 0) gc_ = gdk_gc_new(window_);
    return gc_;
  }

  ////////////////////////////////////////////////////////////////////////////

  void set_foreground(const Gdk_Color& c) {
    gdk_gc_set_foreground(gc(), (Gdk_Color&)c); // cast out const
  }

  void set_background(const Gdk_Color& c) {
    gdk_gc_set_background(gc(), (Gdk_Color&)c); // cast out const
  }

  GdkFont* font();
  
  void set_font(const gchar* s) {
    gdk_gc_set_font(gc(), gdk_font_load(s));
  }

  // I find these to be convenient functions to have available
  gint string_width(const gchar* s) { 
    return gdk_string_width(font(), s);
  }

  gint font_height() {
    GdkFont* f = font();
    return f->ascent + f->descent;
  }

  gint text_width(const gchar*);
  gint text_height(const gchar*);

  // OK, back to the non-font stuff...

  void set_clip_origin(gint x, gint y) {
    gdk_gc_set_clip_origin(gc(),x,y);
  }

  void set_clip_mask(GdkBitmap* mask) {
    gdk_gc_set_clip_mask(gc(),mask);
  }

  void set_clip_rectangle(gint x, gint y, gint w, gint h) {
    GdkRectangle rect;
    rect.x = x;
    rect.y = y;
    rect.width = w;
    rect.height = h;
    gdk_gc_set_clip_rectangle(gc(), &rect);
#ifdef G_ENABLE_DEBUG
    g_print("Gdk_GC: clipping set to %d, %d  %d x %d\n", 
	    x, y, w, h);
#endif
  }

  void set_noclip() {
    gdk_gc_set_clip_mask(gc(), 0);
  }

  // Drawing Functions: GDK_COPY, GDK_INVERT, GDK_XOR
  void set_function(GdkFunction func) {
    gdk_gc_set_function(gc(), func);
  }

  // Fill Styles: GDK_SOLID, GDK_TILED, GDK_STIPPLED, GDK_OPAQUE_STIPPLED
  void set_fill_style(GdkFill fill) {
    gdk_gc_set_fill(gc(), fill);
  }

  void set_tile(GdkPixmap* tile) {
    gdk_gc_set_tile(gc(), tile);
  }

  void set_stipple(GdkPixmap* stip) {
    gdk_gc_set_stipple(gc(), stip);
  }
  
  void set_ts_origin(gint x, gint y) {
    gdk_gc_set_ts_origin(gc(), x, y);
  }

  // Subwindow modes: GDK_CLIP_BY_CHILDREN, GDK_INCLUDE_INFERIORS
  void set_subwindow(GdkSubwindowMode mode) { 
    gdk_gc_set_subwindow(gc(), mode);
  }

  void set_exposures(gint exp) {
    gdk_gc_set_exposures(gc(), exp);
  }

  void set_line_width(gint line_width);
  
  // Line Styles: GDK_LINE_SOLID, GDK_LINE_ON_OFF_DASH, GDK_LINE_DOUBLE_DASH
  void set_line_style(GdkLineStyle line_style);

  // Cap Styles: GDK_CAP_NOT_LAST, GDK_CAP_BUTT, GDK_CAP_ROUND,
  //             GDK_CAP_PROJECTING
  void set_cap_style(GdkCapStyle cap_style);

  // Join Styles: GDK_JOIN_MITER, GDK_JOIN_ROUND, GDK_JOIN_BEVEL
  void set_join_style(GdkJoinStyle join_style);

  // If you want to set everything at one, you can use this...
  void set_line_attributes(gint line_width,
			   GdkLineStyle line_style,
			   GdkCapStyle cap_style,
			   GdkJoinStyle join_style) {
    gdk_gc_set_line_attributes(gc(), line_width, line_style, 
			       cap_style, join_style);
  }

  void set_dashes(gint dash_offset, char* dash_list, gint dash_list_length);

  // Here are the two cases that you use most of the time:
  void set_dashes(gint size); // on and off have same size
  void set_dashes(gint on, gint off);

private:
  GdkWindow* window_;
  GdkGC* gc_;

};

#endif // _INC_GC_H

// $Id: gc.h,v 1.7 1998/07/21 19:41:27 hp Exp $








